@@ -104,14 +104,14 @@ struct SetSQToUndefined
   }
 };
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmconv: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintLossyWarning()
+static void PrintLossyWarning()
 {
   std::cout << "You have selected a lossy compression transfer syntax." << std::endl;
   std::cout << "This will degrade the quality of your input image, and can." << std::endl;
@@ -120,7 +120,7 @@ void PrintLossyWarning()
   std::cout << "WARNING: this mode is very experimental." << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmconv [OPTION] input.dcm output.dcm" << std::endl;
@@ -186,7 +186,7 @@ void PrintHelp()
 }
 
 template <typename T>
-size_t readvector(std::vector<T> &v, const char *str)
+static size_t readvector(std::vector<T> &v, const char *str)
 {
   if( !str ) return 0;
   std::istringstream os( str );
@@ -422,7 +422,7 @@ static bool derives( File & file, const Pixmap& compressed_image )
 return true;
 
 }
-}
+} // end namespace gdcm
 
 int main (int argc, char *argv[])
 {
@@ -941,7 +941,9 @@ int main (int argc, char *argv[])
     gdcm::FileMetaInformation &fmi = file.GetHeader();
 
     const gdcm::TransferSyntax &orits = fmi.GetDataSetTransferSyntax();
-    if( orits != gdcm::TransferSyntax::ExplicitVRLittleEndian && orits != gdcm::TransferSyntax::ImplicitVRLittleEndian && orits != gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian )
+    if( orits != gdcm::TransferSyntax::ExplicitVRLittleEndian
+      && orits != gdcm::TransferSyntax::ImplicitVRLittleEndian
+      && orits != gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian )
       {
       std::cerr << "Sorry input Transfer Syntax not supported for this conversion: " << orits << std::endl;
       return 1;
@@ -956,10 +958,13 @@ int main (int argc, char *argv[])
       {
       ts = gdcm::TransferSyntax::DeflatedExplicitVRLittleEndian;
       }
-    const char *tsuid = gdcm::TransferSyntax::GetTSString( ts );
-    // const char * is ok since padding is \0 anyway...
+    std::string tsuid = gdcm::TransferSyntax::GetTSString( ts );
+    if( tsuid.size() % 2 == 1 )
+      {
+      tsuid.push_back( 0 ); // 0 padding
+      }
     gdcm::DataElement de( gdcm::Tag(0x0002,0x0010) );
-    de.SetByteValue( tsuid, (uint32_t)strlen(tsuid) );
+    de.SetByteValue( &tsuid[0], (uint32_t)tsuid.size() );
     de.SetVR( gdcm::Attribute<0x0002, 0x0010>::GetVR() );
     fmi.Clear();
     fmi.Replace( de );
@@ -1113,7 +1118,7 @@ int main (int argc, char *argv[])
         {
         iig.SetPixelMinMax( iconmin, iconmax );
         }
-      iig.AutoPixelMinMax( iconautominmax );
+      iig.AutoPixelMinMax( iconautominmax ? true : false );
       bool b = iig.Generate();
       if( !b ) return 1;
       const gdcm::IconImage &icon = iig.GetIconImage();
@@ -1275,7 +1280,7 @@ int main (int argc, char *argv[])
     if( lossy )
       {
       if(!quiet)
-      PrintLossyWarning();
+        PrintLossyWarning();
       if( !gdcm::derives( reader.GetFile(), change.PixmapToPixmapFilter::GetOutput() ) )
         {
         std::cerr << "Failed to derives: " << filename << std::endl;
