@@ -404,7 +404,7 @@ struct Data2
 static bool ProcessData( const char *buf, size_t len )
 {
   Data2 data2;
-  size_t s = sizeof(data2);
+  const size_t s = sizeof(data2);
   assert( len >= s); (void)len;
   // VIMDATA2 is generally 2048 bytes, while s = 1786
   // the end is filled with \0 bytes
@@ -854,6 +854,8 @@ static int PrintCSA(const std::string & filename)
   return ret;
 }
 
+
+
 static void PrintVersion()
 {
   std::cout << "gdcmdump: gdcm " << gdcm::Version::GetVersion() << " ";
