@@ -40,6 +40,7 @@
  * Todo: check compat API with jhead
  */
 #include "gdcmFilename.h"
+#include "gdcmImageHelper.h"
 #include "gdcmDirectory.h"
 #include "gdcmMediaStorage.h"
 #include "gdcmSmartPointer.h"
@@ -136,7 +137,7 @@ static void PrintHelp()
   std::cout << "     --pc [01]         Change planar configuration." << std::endl;
   std::cout << "     --pi [str]        Change photometric interpretation." << std::endl;
   std::cout << "     --pf %d,%d,%d     Change pixel format: (BA,BS,HB)." << std::endl;
-  std::cout << "  -s --size %d,%d      Size." << std::endl;
+  std::cout << "  -s --size %d,%d,%d   Size." << std::endl;
   std::cout << "     --offset %ull     Start Offset." << std::endl;
   std::cout << "  -C --sop-class-uid   SOP Class UID (name or value)." << std::endl;
   std::cout << "  -T --study-uid       Study UID." << std::endl;
@@ -194,22 +195,24 @@ static bool AddStudyDateTime(gdcm::DataSet &ds, const char *filename )
   const size_t datelen = 8;
   int res = gdcm::System::GetCurrentDateTime(date);
   if( !res ) return false;
-    {
-    gdcm::DataElement de( gdcm::Tag(0x0008,0x0020) );
-    // Do not copy the whole cstring:
-    de.SetByteValue( date, datelen );
-    de.SetVR( gdcm::Attribute<0x0008,0x0020>::GetVR() );
-    ds.Insert( de );
-    }
+
+  {
+  gdcm::DataElement de( gdcm::Tag(0x0008,0x0020) );
+  // Do not copy the whole cstring:
+  de.SetByteValue( date, datelen );
+  de.SetVR( gdcm::Attribute<0x0008,0x0020>::GetVR() );
+  ds.Insert( de );
+  }
+
   // StudyTime
   const size_t timelen = 6; // get rid of milliseconds
-    {
-    gdcm::DataElement de( gdcm::Tag(0x0008,0x0030) );
-    // Do not copy the whole cstring:
-    de.SetByteValue( date+datelen, timelen );
-    de.SetVR( gdcm::Attribute<0x0008,0x0030>::GetVR() );
-    ds.Insert( de );
-    }
+  {
+  gdcm::DataElement de( gdcm::Tag(0x0008,0x0030) );
+  // Do not copy the whole cstring:
+  de.SetByteValue( date+datelen, timelen );
+  de.SetVR( gdcm::Attribute<0x0008,0x0030>::GetVR() );
+  ds.Insert( de );
+  }
   return AddContentDateTime(ds, filename);
 }
 
@@ -218,6 +221,7 @@ static bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string c
 {
   gdcm::DataSet & ds = writer.GetFile().GetDataSet();
   gdcm::MediaStorage ms = gdcm::MediaStorage::MS_END;
+  gdcm::Pixmap &image = writer.GetPixmap();
   if( sopclassuid )
     {
     // Is it by value or by name ?
@@ -229,35 +233,34 @@ static bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string c
       {
       std::cerr << "not implemented" << std::endl;
       }
-    if( !gdcm::MediaStorage::IsImage(ms) )
-      {
-      std::cerr << "invalid media storage (no pixel data): " << sopclass << std::endl;
-      return false;
-      }
-
-    const char* msstr = gdcm::MediaStorage::GetMSString(ms);
-    if( !msstr )
-      {
-      std::cerr << "problem with media storage: " << sopclass << std::endl;
-      return false;
-      }
-    gdcm::DataElement de( gdcm::Tag(0x0008, 0x0016 ) );
-    de.SetByteValue( msstr, (uint32_t)strlen(msstr) );
-    de.SetVR( gdcm::Attribute<0x0008, 0x0016>::GetVR() );
-    ds.Insert( de );
     }
   else
-    {
-    // FIXME we are copying the default behavior of gdcm.PixmapWriter here:
-    ms = gdcm::MediaStorage::SecondaryCaptureImageStorage;
+    { // guess a default
+    ms = gdcm::ImageHelper::ComputeMediaStorageFromModality(
+      "OT", image.GetNumberOfDimensions(),
+      image.GetPixelFormat(), image.GetPhotometricInterpretation() );
     }
 
-  gdcm::Pixmap &image = writer.GetPixmap();
+  if( !gdcm::MediaStorage::IsImage(ms) )
+    {
+    std::cerr << "invalid media storage (no pixel data): " << sopclass << std::endl;
+    return false;
+    }
   if( ms.GetModalityDimension() < image.GetNumberOfDimensions() )
     {
     std::cerr << "Could not find Modality" << std::endl;
     return false;
     }
+  const char* msstr = gdcm::MediaStorage::GetMSString(ms);
+  if( !msstr )
+    {
+    std::cerr << "problem with media storage: " << sopclass << std::endl;
+    return false;
+    }
+  gdcm::DataElement de( gdcm::Tag(0x0008, 0x0016 ) );
+  de.SetByteValue( msstr, (uint32_t)strlen(msstr) );
+  de.SetVR( gdcm::Attribute<0x0008, 0x0016>::GetVR() );
+  ds.Insert( de );
 
     {
     gdcm::DataElement de( gdcm::Tag(0x0020,0x000d) ); // Study
@@ -276,7 +279,10 @@ static bool AddUIDs(int sopclassuid, std::string const & sopclass, std::string c
   return true;
 }
 
-static bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFragments *sq , gdcm::ImageCodec & jpeg, const char *filename, std::streampos const pos = 0 )
+// Append data to either sq or bv depending whether encapsulated or not
+static bool PopulateSingeFile( gdcm::PixmapWriter & writer,
+  gdcm::SequenceOfFragments *sq , gdcm::ByteValue * bv, gdcm::ImageCodec & jpeg,
+  const char *filename, std::streampos const pos = 0 )
 {
   /*
    * FIXME: when JPEG contains JFIF marker, we should only read them
@@ -328,9 +334,11 @@ static bool PopulateSingeFile( gdcm::PixmapWriter & writer, gdcm::SequenceOfFrag
     }
   else
     {
-    pixeldata.SetByteValue( buf, (uint32_t)len );
+    gdcm::ByteValue frame( buf, (uint32_t)len );
+    bv->Append( frame );
+    pixeldata.SetValue( *bv );
     }
-    delete[] buf;
+  delete[] buf;
   image.SetDataElement( pixeldata );
 
   return true;
@@ -342,18 +350,16 @@ static bool Populate( gdcm::PixmapWriter & writer, gdcm::ImageCodec & jpeg, gdcm
   bool b = true;
   gdcm::Pixmap &image = writer.GetPixmap();
   image.SetNumberOfDimensions( ndim );
+
+  gdcm::SmartPointer<gdcm::SequenceOfFragments> sq = new gdcm::SequenceOfFragments;
+  gdcm::SmartPointer<gdcm::ByteValue> bv = new gdcm::ByteValue;
+  for(; it != filenames.end(); ++it)
+    {
+    b = b && PopulateSingeFile( writer, sq, bv, jpeg, it->c_str(), pos );
+    }
   if( filenames.size() > 1 )
     {
     image.SetNumberOfDimensions( 3 );
-    }
-
-  gdcm::SmartPointer<gdcm::SequenceOfFragments> sq = new gdcm::SequenceOfFragments;
-  for(; it != filenames.end(); ++it)
-    {
-    b = b && PopulateSingeFile( writer, sq, jpeg, it->c_str(), pos );
-    }
-  if( filenames.size() > 1 )
-    {
     image.SetDimension(2,  (unsigned int)filenames.size() );
     }
 
@@ -577,7 +583,7 @@ int main (int argc, char *argv[])
             {
             assert( strcmp(s, "offset") == 0 );
             poffset = 1;
-            start_pos = atoll(optarg);
+            start_pos = (size_t)atoll(optarg);
             }
           //printf (" with arg %s", optarg);
           }
@@ -798,13 +804,13 @@ int main (int argc, char *argv[])
     int ba, bs, hb;
     int n = sscanf( pformatstr.c_str(), "%d,%d,%d", &ba, &bs, &hb );
     if( n != 3 ) return 1;
-    pfref.SetBitsAllocated( ba );
-    pfref.SetBitsStored( bs );
-    pfref.SetHighBit( hb );
+    pfref.SetBitsAllocated( (unsigned short)ba );
+    pfref.SetBitsStored( (unsigned short)bs );
+    pfref.SetHighBit( (unsigned short)hb );
     if( spp )
-      pfref.SetSamplesPerPixel( pixelspp );
+      pfref.SetSamplesPerPixel( (unsigned short)pixelspp );
     if( sign )
-      pfref.SetPixelRepresentation( pixelsign );
+      pfref.SetPixelRepresentation( (unsigned short)pixelsign );
     }
   gdcm::PhotometricInterpretation::PIType refpi = gdcm::PhotometricInterpretation::MONOCHROME2;
   if( pinter )
@@ -823,10 +829,11 @@ int main (int argc, char *argv[])
   //if( !inputextension || !outputextension ) return 1;
   if( inputextension )
     {
-    if(  gdcm::System::StrCaseCmp(inputextension,".raw") == 0
-      || gdcm::System::StrCaseCmp(inputextension,".rawl") == 0 // kakadu convention for raw little endian
-      || gdcm::System::StrCaseCmp(inputextension,".gray") == 0
-      || gdcm::System::StrCaseCmp(inputextension,".rgb") == 0 )
+    if(  gdcm::System::StrCaseCmp(inputextension,".raw") == 0   // watch out that .raw for kakadu means big-endian
+      || gdcm::System::StrCaseCmp(inputextension,".rawl") == 0  // kakadu convention for raw little endian
+      || gdcm::System::StrCaseCmp(inputextension,".gray") == 0  // imagemagick convention
+      || gdcm::System::StrCaseCmp(inputextension,".bin") == 0   // openjp3d convention for raw little endian
+      || gdcm::System::StrCaseCmp(inputextension,".rgb") == 0 ) // imagemagick convention
       {
       if( !size[0] || !size[1] )
         {
@@ -907,9 +914,13 @@ int main (int argc, char *argv[])
       dims[1] = size[1];
       rle.SetDimensions( dims );
       gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
-      if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
+      if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign, spp, pixelspp ) ) return 1;
       rle.SetPixelFormat( pf );
       gdcm::PhotometricInterpretation pi = refpi;
+      if( spp )
+        {
+        if( pixelspp == 3 ) pi = gdcm::PhotometricInterpretation::RGB;
+        }
       rle.SetPhotometricInterpretation( pi );
 
       if( !Populate( writer, rle, filenames ) ) return 1;
@@ -932,7 +943,7 @@ int main (int argc, char *argv[])
       // Let's handle the case where user really wants to specify the data:
       gdcm::PixelFormat pf = gdcm::PixelFormat::UINT8;
       if( !GetPixelFormat( pf, depth, bpp, sign, pixelsign ) ) return 1;
-
+      pnm.SetPixelFormat( pf );
       gdcm::PixmapWriter writer;
       if( !Populate( writer, pnm, filenames ) ) return 1;
       // populate will guess pixel format and photometric inter from file, need
@@ -945,6 +956,10 @@ int main (int argc, char *argv[])
         {
         writer.GetPixmap().SetPhotometricInterpretation( refpi );
         }
+      // HACK
+      if( endian && lsb_msb == "LSB" )
+        writer.GetPixmap().SetTransferSyntax( gdcm::TransferSyntax::ImplicitVRLittleEndian );
+
       if( !AddUIDs(sopclassuid, sopclass, study_uid, series_uid, writer ) ) return 1;
 
       writer.SetFileName( outfilename );
