@@ -52,7 +52,7 @@
 #include <sys/stat.h>
 
 
-int checkmagick(unsigned char *input)
+static int checkmagick(unsigned char *input)
 {
   if( input[128+0] == 'D'
    && input[128+1] == 'I'
@@ -64,7 +64,7 @@ int checkmagick(unsigned char *input)
   return 0;
 }
 
-int checkdeflated(const char *name)
+static int checkdeflated(const char *name)
 {
   int ret;
   unsigned char *source;
@@ -183,7 +183,7 @@ int checkdeflated(const char *name)
 }
 
 #ifdef GDCM_USE_SYSTEM_POPPLER
-std::string getInfoDate(Dict *infoDict, const char *key)
+static std::string getInfoDate(Dict *infoDict, const char *key)
 {
   Object obj;
   char *s;
@@ -244,7 +244,7 @@ std::string getInfoDate(Dict *infoDict, const char *key)
   return out;
 }
 
-std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
+static std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
 {
   Object obj;
   GooString *s1;
@@ -292,14 +292,14 @@ std::string getInfoString(Dict *infoDict, const char *key, UnicodeMap *uMap)
 #endif
 
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcminfo: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcminfo [OPTION]... FILE..." << std::endl;
@@ -331,7 +331,7 @@ void PrintHelp()
   int checkcompression = 0;
   int md5sum = 0;
 
-int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
+static int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
 {
   (void)defs;
   if( deflated )
@@ -471,7 +471,11 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
         moddate      = getInfoDate(  info.getDict(), "ModDate"       );
         info.free();
         }
+#ifdef LIBPOPPLER_CATALOG_HAS_STRUCTTREEROOT
+      const char *tagged = doc->getStructTreeRoot() ? "yes" : "no";
+#else
       const char *tagged = doc->getStructTreeRoot()->isDict() ? "yes" : "no";
+#endif
       int pages = doc->getNumPages();
       const char *encrypted = doc->isEncrypted() ? "yes" : "no";
       //  printf("yes (print:%s copy:%s change:%s addNotes:%s)\n",
@@ -487,7 +491,7 @@ int ProcessOneFile( std::string const & filename, gdcm::Defs const & defs )
 #ifdef LIBPOPPLER_PDFDOC_HAS_PDFVERSION
       float pdfversion = doc->getPDFVersion();
 #else
-      float pdfversion = doc->getPDFMajorVersion() + 0.1 * doc->getPDFMinorVersion();
+      const double pdfversion = doc->getPDFMajorVersion() + 0.1 * doc->getPDFMinorVersion();
 #endif
 
 
