@@ -40,14 +40,14 @@
 #include "gdcmPrinter.h"
 
 
-void PrintVersion()
+static void PrintVersion()
 {
   std::cout << "gdcmscu: gdcm " << gdcm::Version::GetVersion() << " ";
   const char date[] = "$Date$";
   std::cout << date << std::endl;
 }
 
-void PrintHelp()
+static void PrintHelp()
 {
   PrintVersion();
   std::cout << "Usage: gdcmscu [OPTION]...[OPERATION]...HOSTNAME...[PORT]..." << std::endl;
@@ -115,7 +115,7 @@ void PrintHelp()
   std::cout << "DICOM Character Set: [" << s << "]" << std::endl;
 }
 
-void PrintQueryHelp(int inFindPatientRoot)
+static void PrintQueryHelp(int inFindPatientRoot)
 {
   gdcm::BaseRootQuery* theBase;
   if (inFindPatientRoot)
@@ -591,7 +591,7 @@ int main(int argc, char *argv[])
 
     if( !portscp )
       {
-      std::cerr << "Need to set explicitely port number for SCP association"
+      std::cerr << "Need to set explicitly port number for SCP association"
         " --port-scp" << std::endl;
       //std::cerr << "Move failed." << std::endl;
       return 1;
