@@ -134,6 +134,7 @@ static const char *MSStrings[] = {
   "1.2.392.200036.9125.1.1.2",  // FujiPrivateCRImageStorage
   "1.2.840.10008.5.1.4.1.1.77.1.5.1", // OphthalmicPhotography8BitImageStorage
   "1.2.840.10008.5.1.4.1.1.77.1.5.4", // OphthalmicTomographyImageStorage
+  "1.2.840.10008.5.1.4.1.1.77.1.2",   // VL Microscopic Image Storage
   0
 };
 
@@ -310,6 +311,7 @@ static const MSModalityType MSModalityTypes[] = {
   {"CR", 2, 0},// FujiPrivateCRImageStorage
   {"OP", 2, 0},// OphthalmicPhotography8BitImageStorage
   {"OPT", 3, 0},// OphthalmicTomographyImageStorage
+  {"GM", 3, 0},// VLMicroscopicImageStorage
 
   {NULL, 0, 0} //MS_END
 };
@@ -351,7 +353,7 @@ unsigned int MediaStorage::GetModalityDimension() const
   return MSModalityTypes[MSField].Dimension;
 }
 
-void MediaStorage::GuessFromModality(const char *modality, unsigned int dim)
+void MediaStorage::GuessFromModality(const char *modality, unsigned int dim )
 {
   // no default value is set, it is up to the user to decide initial value
   if( !modality || !dim ) return;
@@ -513,7 +515,7 @@ bool MediaStorage::SetFromFile(File const &file)
   /*
    * DICOMDIR usually have group 0002 present, but no 0008,0016 (doh!)
    * So we first check in header, if found, assumed it is ok (we should
-   * check that consistant with 0008,0016 ...)
+   * check that consistent with 0008,0016 ...)
    * A lot of DICOM image file are still missing the group header
    * this is why we check 0008,0016, and to preserve compat with ACR-NEMA
    * we also check Modality element to guess a fake Media Storage UID
