@@ -210,6 +210,9 @@ namespace details
       void ReadCommonWithLength(std::istream & is, VL & length) const
         {
         m_dataSet.template ReadWithLength<T1,T2>(is,length);
+        // manually set eofbit:
+        // https://groups.google.com/forum/?fromgroups#!topic/comp.lang.c++/yTW4ESh1IL8
+        is.setstate( std::ios::eofbit );
         }
     static void Check(bool b, std::istream &stream)
       {
@@ -239,7 +242,7 @@ namespace details
     template<class T1, class T2>
       void ReadCommonWithLength(std::istream & is, VL & length) const
         {
-        m_dataSet.template ReadUpToTagWithLength<T1,T2>(is,m_tag,length);
+        m_dataSet.template ReadUpToTagWithLength<T1,T2>(is,m_tag,m_skipTags,length);
         }
     static void Check(bool , std::istream &)  {}
   };
@@ -249,22 +252,49 @@ namespace details
   private:
     DataSet & m_dataSet;
     std::set<Tag> const & m_tags;
+    bool m_readvalues;
   public:
-    ReadSelectedTagsCaller(DataSet &ds, std::set<Tag> const & tags)
+    ReadSelectedTagsCaller(DataSet &ds, std::set<Tag> const & tags, const bool readvalues)
       :
-    m_dataSet(ds),m_tags(tags)
+    m_dataSet(ds),m_tags(tags),m_readvalues(readvalues)
     {
     }
 
     template<class T1, class T2>
     void ReadCommon(std::istream & is) const
     {
-      m_dataSet.template ReadSelectedTags<T1,T2>(is,m_tags);
+      m_dataSet.template ReadSelectedTags<T1,T2>(is,m_tags,m_readvalues);
     }
     template<class T1, class T2>
     void ReadCommonWithLength(std::istream & is, VL & length) const
     {
-      m_dataSet.template ReadSelectedTagsWithLength<T1,T2>(is,m_tags,length);
+      m_dataSet.template ReadSelectedTagsWithLength<T1,T2>(is,m_tags,length,m_readvalues);
+    }
+    static void Check(bool , std::istream &)  {}
+  };
+
+  class ReadSelectedPrivateTagsCaller
+  {
+  private:
+    DataSet & m_dataSet;
+    std::set<PrivateTag> const & m_groups;
+    bool m_readvalues;
+  public:
+    ReadSelectedPrivateTagsCaller(DataSet &ds, std::set<PrivateTag> const & groups, const bool readvalues)
+      :
+    m_dataSet(ds),m_groups(groups),m_readvalues(readvalues)
+    {
+    }
+
+    template<class T1, class T2>
+    void ReadCommon(std::istream & is) const
+    {
+      m_dataSet.template ReadSelectedPrivateTags<T1,T2>(is,m_groups,m_readvalues);
+    }
+    template<class T1, class T2>
+    void ReadCommonWithLength(std::istream & is, VL & length) const
+    {
+      m_dataSet.template ReadSelectedPrivateTagsWithLength<T1,T2>(is,m_groups,length,m_readvalues);
     }
     static void Check(bool , std::istream &)  {}
   };
@@ -282,9 +312,15 @@ bool Reader::ReadUpToTag(const Tag & tag, std::set<Tag> const & skiptags)
   return InternalReadCommon(caller);
 }
 
-bool Reader::ReadSelectedTags( std::set<Tag> const & selectedTags )
+bool Reader::ReadSelectedTags( std::set<Tag> const & selectedTags, bool readvalues )
 {
-  details::ReadSelectedTagsCaller caller(F->GetDataSet(), selectedTags);
+  details::ReadSelectedTagsCaller caller(F->GetDataSet(), selectedTags,readvalues);
+  return InternalReadCommon(caller);
+}
+
+bool Reader::ReadSelectedPrivateTags( std::set<PrivateTag> const & selectedPTags, bool readvalues )
+{
+  details::ReadSelectedPrivateTagsCaller caller(F->GetDataSet(), selectedPTags,readvalues);
   return InternalReadCommon(caller);
 }
 
@@ -385,7 +421,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
   if( ts == TransferSyntax::DeflatedExplicitVRLittleEndian )
     {
 #if 0
-  std::ofstream out( "/tmp/deflate.raw");
+  std::ofstream out( "/tmp/deflate.raw", std::ios::binary );
   out << is.rdbuf();
   out.close();
 #endif
@@ -412,6 +448,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
         // LIBIDO-16-ACR_NEMA-Volume.dcm
         //F->GetDataSet().ReadUpToTag<ImplicitDataElement,SwapperDoOp>(is,tag, skiptags);
         //caller.template ReadCommon<ImplicitDataElement,SwapperDoOp>(is);
+        gdcmErrorMacro( "VirtualBigEndianNotHandled" );
         throw "Virtual Big Endian Implicit is not defined by DICOM";
         }
       else
@@ -433,19 +470,20 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
           {
           std::streampos start = is.tellg();
           is.seekg( 0, std::ios::end);
-	  std::streampos end = is.tellg();
-	  std::streamoff theOffset = end-start;
-	  assert (theOffset > 0 || (uint32_t)theOffset < std::numeric_limits<uint32_t>::max());
-	  VL l = (uint32_t)(theOffset);
+          std::streampos end = is.tellg();
+          assert( !is.eof() );
+          assert( is.good() );
+          std::streamoff theOffset = end-start;
+          assert (theOffset > 0 || (uint32_t)theOffset < std::numeric_limits<uint32_t>::max());
+          VL l = (uint32_t)(theOffset);
           is.seekg( start, std::ios::beg );
-          //F->GetDataSet().ReadUpToTagWithLength<ImplicitDataElement,SwapperNoOp>(is, tag, l);
+          assert( is.good() );
+          assert( !is.eof() );
           caller.template ReadCommonWithLength<ImplicitDataElement,SwapperNoOp>(is,l);
-          is.peek();
           }
         }
       else
         {
-        //F->GetDataSet().ReadUpToTag<ExplicitDataElement,SwapperNoOp>(is,tag, skiptags);
         caller.template ReadCommon<ExplicitDataElement,SwapperNoOp>(is);
         }
       }
@@ -653,6 +691,11 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
         caller.template ReadCommon<ExplicitImplicitDataElement,SwapperNoOp>(is);
         // This file can only be rewritten as implicit...
         }
+      else
+        {
+        gdcmDebugMacro( "No way this is DICOM" );
+        success = false;
+        }
       }
 #else
     gdcmDebugMacro( ex.what() );
@@ -661,7 +704,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
     }
   catch( Exception &ex )
     {
-	(void)ex;  //to avoid unreferenced variable warning on release
+    (void)ex;  //to avoid unreferenced variable warning on release
     gdcmDebugMacro( ex.what() );
     success = false;
     }
@@ -676,7 +719,7 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
     }
   catch( Exception &ex )
     {
-	(void)ex;  //to avoid unreferenced variable warning on release
+    (void)ex;  //to avoid unreferenced variable warning on release
     gdcmDebugMacro( ex.what() );
     success = false;
     }
@@ -705,11 +748,12 @@ bool Reader::InternalReadCommon(const T_Caller &caller)
 
 // This function re-implements code from:
 // http://www.dclunie.com/medical-image-faq/html/part2.html#DICOMTransferSyntaxDetermination
-// The above code does not work well for random file. It implicitly assumes we are trying
-// to read a DICOM file in the first place, while our goal is indeed to detect whether or
-// not the file can be assimilated as DICOM. So we extended it.
-// Of course this function only returns a 'maybe DICOM', since we are not guaranteed that
-// the stream is not truncated, but this is outside the scope of this function.
+// The above code does not work well for random file. It implicitly assumes we
+// are trying to read a DICOM file in the first place, while our goal is indeed
+// to detect whether or not the file can be assimilated as DICOM. So we
+// extended it.  Of course this function only returns a 'maybe DICOM', since we
+// are not guaranteed that the stream is not truncated, but this is outside the
+// scope of this function.
 bool Reader::CanRead() const
 {
   // fastpath
@@ -728,7 +772,7 @@ bool Reader::CanRead() const
 
   // Start overhead for backward compatibility
   bool bigendian = false;
-  bool explicitvr	= false;
+  bool explicitvr = false;
   is.clear();
   //is.seekg(0, std::ios::end);
   //std::streampos filelen = is.tellg();
@@ -965,5 +1009,10 @@ void Reader::SetFileName(const char *filename)
     }
 }
 
+size_t Reader::GetStreamCurrentPosition() const
+{
+  return GetStreamPtr()->tellg();
+}
+
 
 } // end namespace gdcm
