@@ -83,11 +83,11 @@ public:
   BasicOffsetTable &GetTable() { return Table; }
 
 template <typename TSwap>
-std::istream& Read(std::istream &is)
+std::istream& Read(std::istream &is, bool readvalues = true)
 {
   assert( SequenceLengthField.IsUndefined() );
   ReadPreValue<TSwap>(is);
-  return ReadValue<TSwap>(is);
+  return ReadValue<TSwap>(is, readvalues);
 }
 
 template <typename TSwap>
@@ -102,16 +102,21 @@ std::istream& ReadPreValue(std::istream &is)
     }
   catch(...)
     {
+    // throw "SIEMENS Icon thingy";
     // Bug_Siemens_PrivateIconNoItem.dcm
     // First thing first let's rewind
     is.seekg(-4, std::ios::cur);
+    // FF D8 <=> Start of Image (SOI) marker
+    // FF E0 <=> APP0 Reserved for Application Use
     if ( Table.GetTag() == Tag(0xd8ff,0xe0ff) )
       {
+      Table = BasicOffsetTable(); // clear up stuff
+      //Table.SetByteValue( "", 0 );
       Fragment frag;
-      is.seekg( 8340, std::ios::cur );
-      char dummy[8340];
-      frag.SetByteValue( dummy, 8340 - Table.GetLength() - 16 );
-      Fragments.push_back( frag );
+      if( FillFragmentWithJPEG( frag, is ) )
+        {
+        Fragments.push_back( frag );
+        }
       return is;
       }
     else
@@ -124,7 +129,7 @@ std::istream& ReadPreValue(std::istream &is)
 }
 
 template <typename TSwap>
-std::istream& ReadValue(std::istream &is)
+std::istream& ReadValue(std::istream &is, bool /*readvalues*/)
 {
   const Tag seqDelItem(0xfffe,0xe0dd);
   // not used for now...
@@ -133,7 +138,7 @@ std::istream& ReadValue(std::istream &is)
     {
     while( frag.Read<TSwap>(is) && frag.GetTag() != seqDelItem )
       {
-      gdcmDebugMacro( "Frag: " << frag );
+      //gdcmDebugMacro( "Frag: " << frag );
       Fragments.push_back( frag );
       }
     assert( frag.GetTag() == seqDelItem && frag.GetVL() == 0 );
@@ -313,6 +318,9 @@ private:
   VL SequenceLengthField;
   /// \brief Vector of Sequence Fragments
   FragmentVector Fragments;
+
+private:
+  bool FillFragmentWithJPEG( Fragment & frag, std::istream & is );
 };
 
 /**
