@@ -70,7 +70,7 @@ bool GetOriginValueFromSequence(const DataSet& ds, const Tag& tfgs, std::vector<
   if( !subds2.FindDataElement(tps) ) return false;
   const DataElement &de = subds2.GetDataElement( tps );
   //assert( bv );
-  gdcm::Attribute<0x0020,0x0032> at;
+  Attribute<0x0020,0x0032> at;
   at.SetFromDataElement( de );
   //at.Print( std::cout );
   ori.push_back( at.GetValue(0) );
@@ -103,7 +103,7 @@ bool GetDirectionCosinesValueFromSequence(const DataSet& ds, const Tag& tfgs, st
   if( !subds2.FindDataElement(tps) ) return false;
   const DataElement &de = subds2.GetDataElement( tps );
   //assert( bv );
-  gdcm::Attribute<0x0020,0x0037> at;
+  Attribute<0x0020,0x0037> at;
   at.SetFromDataElement( de );
   dircos.push_back( at.GetValue(0) );
   dircos.push_back( at.GetValue(1) );
@@ -138,7 +138,7 @@ bool GetInterceptSlopeValueFromSequence(const DataSet& ds, const Tag& tfgs, std:
     if( !subds2.FindDataElement(tps) ) return false;
     const DataElement &de = subds2.GetDataElement( tps );
     //assert( bv );
-    gdcm::Attribute<0x0028,0x1052> at;
+    Attribute<0x0028,0x1052> at;
     at.SetFromDataElement( de );
     //at.Print( std::cout );
     intslope.push_back( at.GetValue() );
@@ -149,7 +149,7 @@ bool GetInterceptSlopeValueFromSequence(const DataSet& ds, const Tag& tfgs, std:
     if( !subds2.FindDataElement(tps) ) return false;
     const DataElement &de = subds2.GetDataElement( tps );
     //assert( bv );
-    gdcm::Attribute<0x0028,0x1053> at;
+    Attribute<0x0028,0x1053> at;
     at.SetFromDataElement( de );
     //at.Print( std::cout );
     intslope.push_back( at.GetValue() );
@@ -202,9 +202,9 @@ bool ComputeZSpacingFromIPP(const DataSet &ds, double &zspacing)
 
   // For each item
   std::vector<double> distances;
-  gdcm::SequenceOfItems::SizeType nitems = sqi->GetNumberOfItems();
+  SequenceOfItems::SizeType nitems = sqi->GetNumberOfItems();
   std::vector<double> dircos_subds2; dircos_subds2.resize(6);
-  for(gdcm::SequenceOfItems::SizeType i0 = 1; i0 <= nitems; ++i0)
+  for(SequenceOfItems::SizeType i0 = 1; i0 <= nitems; ++i0)
     {
     const Item &item = sqi->GetItem(i0);
     const DataSet & subds = item.GetNestedDataSet();
@@ -258,7 +258,7 @@ bool ComputeZSpacingFromIPP(const DataSet &ds, double &zspacing)
 
   if( !timeseries )
     {
-    // Check spacing is consistant:
+    // Check spacing is consistent:
     const double ZTolerance = 1e-3; // ??? FIXME
     prev = distances[0];
     for(unsigned int i = 1; i < nitems; ++i)
@@ -307,7 +307,7 @@ bool GetSpacingValueFromSequence(const DataSet& ds, const Tag& tfgs, std::vector
   if( !subds2.FindDataElement(tps) ) return false;
   const DataElement &de = subds2.GetDataElement( tps );
   //assert( bv );
-  gdcm::Attribute<0x0028,0x0030> at;
+  Attribute<0x0028,0x0030> at;
   at.SetFromDataElement( de );
   //at.Print( std::cout );
   sp.push_back( at.GetValue(1) );
@@ -322,7 +322,7 @@ bool GetSpacingValueFromSequence(const DataSet& ds, const Tag& tfgs, std::vector
   const Tag tst(0x0018,0x0050);
   if( !subds2.FindDataElement(tst) ) return false;
   const DataElement &de2 = subds2.GetDataElement( tst );
-  gdcm::Attribute<0x0018,0x0050> at2;
+  Attribute<0x0018,0x0050> at2;
   at2.SetFromDataElement( de2 );
   //at2.Print( std::cout );
   sp.push_back( at2.GetValue(0) );
@@ -370,8 +370,8 @@ bool GetUltraSoundSpacingValueFromSequence(const DataSet& ds, std::vector<double
   const DataSet & subds = item.GetNestedDataSet();
   //  (0018,602c) FD 0.002                                    #   8, 1 PhysicalDeltaX
   //  (0018,602e) FD 0.002                                    #   8, 1 PhysicalDeltaY
-  gdcm::Attribute<0x0018,0x602c> at1;
-  gdcm::Attribute<0x0018,0x602e> at2;
+  Attribute<0x0018,0x602c> at1;
+  Attribute<0x0018,0x602e> at2;
   const DataElement &de1 = subds.GetDataElement( at1.GetTag() );
   at1.SetFromDataElement( de1 );
   assert( at1.GetNumberOfValues() == 1 );
@@ -770,7 +770,7 @@ std::vector<unsigned int> ImageHelper::GetDimensionsValue(const File& f)
         {
         const DataElement &de = ds.GetDataElement( at.GetTag() );
         // SIEMENS_MAGNETOM-12-MONO2-Uncompressed.dcm picks VR::SS instead...
-        if( de.GetVR().Compatible( at.GetVR() ) || de.GetVR() == VR::INVALID )
+        if( at.GetVR().Compatible( de.GetVR() ) )
           {
           at.SetFromDataSet( ds );
           int imagedimensions = at.GetValue();
@@ -881,6 +881,7 @@ std::vector<double> ImageHelper::GetRescaleInterceptSlopeValue(File const & f)
     ms == MediaStorage::RTDoseStorage
   )
     {
+    // TODO. Should I check FrameIncrementPointer ? (0028,0009) AT (3004,000c)
     Attribute<0x3004,0x000e> gridscaling = { 0 };
     gridscaling.SetFromDataSet( ds );
     interceptslope[0] = 0;
@@ -919,6 +920,8 @@ Tag ImageHelper::GetSpacingTagFromMediaStorage(MediaStorage const &ms)
   case MediaStorage::PETImageStorage:
   case MediaStorage::GeneralElectricMagneticResonanceImageStorage:
   case MediaStorage::PhilipsPrivateMRSyntheticImageStorage:
+  case MediaStorage::VLPhotographicImageStorage: // VL Image IOD
+  case MediaStorage::VLMicroscopicImageStorage:
     // (0028,0030) DS [2.0\2.0]                                #   8, 2 PixelSpacing
     t = Tag(0x0028,0x0030);
     break;
@@ -1129,21 +1132,36 @@ std::vector<double> ImageHelper::GetSpacingValue(File const & f)
         // The spacing is something like that: [0.2\0\0.200000]
         // I would need to throw an expection that VM is not compatible
         el.SetLength( entry.GetVM().GetLength() * entry.GetVR().GetSizeof() );
-        el.Read( ss );
-        assert( el.GetLength() == 2 );
-        for(unsigned int i = 0; i < el.GetLength(); ++i)
+        std::string::size_type found = s.find('\\');
+        if( found != std::string::npos )
           {
-          if( el.GetValue(i) )
+          el.Read( ss );
+          assert( el.GetLength() == 2 );
+          for(unsigned int i = 0; i < el.GetLength(); ++i)
             {
-            sp.push_back( el.GetValue(i) );
-            }
-          else
-            {
-            gdcmWarningMacro( "Cant have a spacing of 0" );
-            sp.push_back( 1.0 );
+            if( el.GetValue(i) )
+              {
+              sp.push_back( el.GetValue(i) );
+              }
+            else
+              {
+              gdcmWarningMacro( "Cant have a spacing of 0" );
+              sp.push_back( 1.0 );
+              }
             }
+          std::swap( sp[0], sp[1]);
+          }
+        else
+          {
+          double singleval;
+          ss >> singleval;
+          if( singleval == 0.0 )
+            {
+            singleval = 1.0;
+            }
+          sp.push_back( singleval );
+          sp.push_back( singleval );
           }
-        std::swap( sp[0], sp[1]);
         assert( sp.size() == (unsigned int)entry.GetVM() );
         }
       break;
@@ -1191,7 +1209,7 @@ std::vector<double> ImageHelper::GetSpacingValue(File const & f)
       const Dicts &dicts = g.GetDicts();
       const DictEntry &entry = dicts.GetDictEntry(de.GetTag());
       const VR & vr = entry.GetVR();
-      assert( de.GetVR() == vr || de.GetVR() == VR::INVALID );
+      assert( de.GetVR() == vr || de.GetVR() == VR::INVALID || de.GetVR() == VR::UN );
       if( entry.GetVM() == VM::VM1 )
         {
         switch(vr)
@@ -1239,10 +1257,10 @@ std::vector<double> ImageHelper::GetSpacingValue(File const & f)
   else if( ds.FindDataElement( Tag(0x0028,0x0009) ) ) // Frame Increment Pointer
     {
     const DataElement& de = ds.GetDataElement( Tag(0x0028,0x0009) );
-    gdcm::Attribute<0x0028,0x0009,VR::AT,VM::VM1> at;
+    Attribute<0x0028,0x0009,VR::AT,VM::VM1> at;
     at.SetFromDataElement( de );
     assert( ds.FindDataElement( at.GetTag() ) );
-    if( ds.FindDataElement( at.GetTag() ) )
+    if( ds.FindDataElement( at.GetValue() ) )
       {
 /*
 $ dcmdump D_CLUNIE_NM1_JPLL.dcm" | grep 0028,0009
@@ -1276,6 +1294,12 @@ $ dcmdump D_CLUNIE_NM1_JPLL.dcm" | grep 0028,0009
         sp.push_back( 1.0 );
         }
       }
+    else
+      {
+      gdcmErrorMacro( "Tag: " << at.GetTag() << " was found to point to missing"
+        "Tag: " << at.GetValue() << " default to 1.0." );
+      sp.push_back( 1.0 );
+      }
     }
   else
     {
@@ -1565,7 +1589,7 @@ void SetDataElementInSQAsItemNumber(DataSet & ds, DataElement const & de, Tag co
     Item &item2 = sqi->GetItem(1);
     DataSet &subds2 = item2.GetNestedDataSet();
 
-    //gdcm::Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
+    //Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
     //ipp.SetValue( origin[0], 0);
     //ipp.SetValue( origin[1], 1);
     //ipp.SetValue( origin[2], 2);
@@ -1583,8 +1607,9 @@ void ImageHelper::SetOriginValue(DataSet & ds, const Image & image)
 
   if( ms == MediaStorage::SecondaryCaptureImageStorage )
     {
-    Tag ipp(0x0020,0x0032);
-    ds.Remove( ipp );
+    // https://sourceforge.net/p/gdcm/bugs/322/
+    // default behavior is simply to pass
+    return;
     }
 
   // FIXME Hardcoded
@@ -1614,7 +1639,7 @@ void ImageHelper::SetOriginValue(DataSet & ds, const Image & image)
 
     const Tag tfgs(0x5200,0x9230);
 
-    gdcm::Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
+    Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
     double zspacing = image.GetSpacing(2);
     unsigned int dimz = image.GetDimension(2);
     const double *cosines = image.GetDirectionCosines();
@@ -1643,7 +1668,7 @@ void ImageHelper::SetOriginValue(DataSet & ds, const Image & image)
     }
 
   // Image Position (Patient)
-  gdcm::Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
+  Attribute<0x0020,0x0032> ipp = {{0,0,0}}; // default value
   ipp.SetValue( origin[0], 0);
   ipp.SetValue( origin[1], 1);
   ipp.SetValue( origin[2], 2);
@@ -1659,8 +1684,8 @@ void ImageHelper::SetDirectionCosinesValue(DataSet & ds, const std::vector<doubl
 
   if( ms == MediaStorage::SecondaryCaptureImageStorage )
     {
-    Tag iop(0x0020,0x0037);
-    ds.Remove( iop );
+    // https://sourceforge.net/p/gdcm/bugs/322/
+    // default behavior is simply to pass
     return;
     }
 
@@ -1678,7 +1703,7 @@ void ImageHelper::SetDirectionCosinesValue(DataSet & ds, const std::vector<doubl
     }
 
   // Image Orientation (Patient)
-  gdcm::Attribute<0x0020,0x0037> iop = {{1,0,0,0,1,0}}; // default value
+  Attribute<0x0020,0x0037> iop = {{1,0,0,0,1,0}}; // default value
 
   assert( dircos.size() == 6 );
   DirectionCosines dc( &dircos[0] );
@@ -1860,10 +1885,20 @@ void ImageHelper::SetRescaleInterceptSlopeValue(File & f, const Image & img)
 
   if( ms == MediaStorage::RTDoseStorage )
     {
+    if( img.GetIntercept() != 0 )
+      {
+      gdcmErrorMacro( "Cannot have an intercept value for RTDOSE, only Scaling allowed" );
+      return;
+      }
     Attribute<0x3004,0x00e> at2;
     at2.SetValue( img.GetSlope() );
     ds.Replace( at2.GetAsDataElement() );
 
+    Attribute<0x0028,0x0009> framePointer;
+    framePointer.SetNumberOfValues(1);
+    framePointer.SetValue( Tag(0x3004,0x000C) );
+    ds.Replace( framePointer.GetAsDataElement() );
+
     return;
     }
 
@@ -1891,7 +1926,7 @@ void ImageHelper::SetRescaleInterceptSlopeValue(File & f, const Image & img)
     else
       {
       // In case user decide to override the default:
-      ds.Insert( at3.GetAsDataElement() );
+      ds.ReplaceEmpty( at3.GetAsDataElement() );
       }
     }
 }
@@ -2174,4 +2209,104 @@ const ByteValue* ImageHelper::GetPointerFromElement(Tag const &tag, const File&
   return 0;
 }
 
+MediaStorage ImageHelper::ComputeMediaStorageFromModality(const char *modality,
+  unsigned int dimension, PixelFormat const & pixeltype,
+  PhotometricInterpretation const & pi,
+  double intercept , double slope
+  )
+{
+  MediaStorage ms = MediaStorage::SecondaryCaptureImageStorage;
+  ms.GuessFromModality(modality, dimension );
+
+  // refine for SC family
+  if( dimension != 2 &&
+    (ms == MediaStorage::SecondaryCaptureImageStorage // dim 2
+  || ms == MediaStorage::MultiframeSingleBitSecondaryCaptureImageStorage ) // dim 3
+  )
+    {
+    // A.8.3.4 Multi-frame Grayscale Byte SC Image IOD Content Constraints
+/*
+- Samples per Pixel (0028,0002) shall be 1
+- Photometric Interpretation (0028,0004) shall be MONOCHROME2
+- Bits Allocated (0028,0100) shall be 8
+- Bits Stored (0028,0101) shall be 8
+- High Bit (0028,0102) shall be 7
+- Pixel Representation (0028,0103) shall be 0
+- Planar Configuration (0028,0006) shall not be present
+*/
+    if( dimension == 3 &&
+      pixeltype.GetSamplesPerPixel() == 1 &&
+      pi == PhotometricInterpretation::MONOCHROME2 &&
+      pixeltype.GetBitsAllocated() == 8 &&
+      pixeltype.GetBitsStored() == 8 &&
+      pixeltype.GetHighBit() == 7 &&
+      pixeltype.GetPixelRepresentation() == 0
+    )
+      {
+      ms = MediaStorage::MultiframeGrayscaleByteSecondaryCaptureImageStorage;
+      if( intercept != 0 || slope != 1 )
+        {
+        // Table C.8-25b SC MULTI-FRAME IMAGE MODULE ATTRIBUTES
+        // Note: This specifies an identity Modality LUT transformation.
+        gdcmErrorMacro( "Cannot have shift/scale" );
+        return MediaStorage::MS_END;
+        }
+      }
+    else if( dimension == 3 &&
+      pixeltype.GetSamplesPerPixel() == 1 &&
+      pi == PhotometricInterpretation::MONOCHROME2 &&
+      pixeltype.GetBitsAllocated() == 1 &&
+      pixeltype.GetBitsStored() == 1 &&
+      pixeltype.GetHighBit() == 0 &&
+      pixeltype.GetPixelRepresentation() == 0
+    )
+      {
+      ms = MediaStorage::MultiframeSingleBitSecondaryCaptureImageStorage;
+      if( intercept != 0 || slope != 1 )
+        {
+        gdcmDebugMacro( "Cannot have shift/scale" );
+        return MediaStorage::MS_END;
+        }
+      }
+    else if( dimension == 3 &&
+      pixeltype.GetSamplesPerPixel() == 1 &&
+      pi == PhotometricInterpretation::MONOCHROME2 &&
+      pixeltype.GetBitsAllocated() == 16 &&
+      pixeltype.GetBitsStored() <= 16 && pixeltype.GetBitsStored() >= 9 &&
+      pixeltype.GetHighBit() == pixeltype.GetBitsStored() - 1 &&
+      pixeltype.GetPixelRepresentation() == 0
+    )
+      {
+      ms = MediaStorage::MultiframeGrayscaleWordSecondaryCaptureImageStorage;
+      if( intercept != 0 || slope != 1 )
+        {
+        gdcmDebugMacro( "Cannot have shift/scale" );
+        return MediaStorage::MS_END;
+        }
+      }
+    else if( dimension == 3 &&
+      pixeltype.GetSamplesPerPixel() == 3 &&
+      pi == PhotometricInterpretation::RGB &&
+      pixeltype.GetBitsAllocated() == 8 &&
+      pixeltype.GetBitsStored() == 8 &&
+      pixeltype.GetHighBit() == 7 &&
+      pixeltype.GetPixelRepresentation() == 0
+    )
+      {
+      ms = MediaStorage::MultiframeTrueColorSecondaryCaptureImageStorage;
+      if( intercept != 0 || slope != 1 )
+        {
+        gdcmDebugMacro( "Cannot have shift/scale" );
+        return MediaStorage::MS_END;
+        }
+      }
+    else
+      {
+      gdcmDebugMacro( "Cannot handle Multi Frame image in SecondaryCaptureImageStorage" );
+      return MediaStorage::MS_END;
+      }
+    }
+  return ms;
 }
+
+} // end namespace gdcm
