@@ -27,7 +27,7 @@
 namespace gdcm
 {
 
-JPEGCodec::JPEGCodec():BitSample(0),Lossless(true),Quality(100)
+JPEGCodec::JPEGCodec():BitSample(0)/*,Lossless(true)*/,Quality(100)
 {
   Internal = NULL;
 }
@@ -50,12 +50,12 @@ double JPEGCodec::GetQuality() const
 
 void JPEGCodec::SetLossless(bool l)
 {
-  Lossless = l;
+  LossyFlag = !l;
 }
 
 bool JPEGCodec::GetLossless() const
 {
-  return Lossless;
+  return !LossyFlag;
 }
 
 bool JPEGCodec::CanDecode(TransferSyntax const &ts) const
@@ -136,6 +136,8 @@ void JPEGCodec::SetBitSample(int bit)
     Internal->SetDimensions( this->GetDimensions() );
     Internal->SetPlanarConfiguration( this->GetPlanarConfiguration() );
     Internal->SetPhotometricInterpretation( this->GetPhotometricInterpretation() );
+    Internal->SetLossless( this->GetLossless() );
+    Internal->SetQuality( this->GetQuality() );
     Internal->ImageCodec::SetPixelFormat( this->ImageCodec::GetPixelFormat() );
     //Internal->SetNeedOverlayCleanup( this->AreOverlaysInPixelData() );
     }
@@ -185,7 +187,7 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
         if( i >= this->GetDimensions()[2] )
           {
           // JPEGInvalidSecondFrag.dcm
-          assert( nfrags == this->GetNumberOfDimensions() ); // sentinel
+          assert( nfrags == this->GetNumberOfDimensions() ); (void)nfrags; // sentinel
           gdcmWarningMacro( "Invalid JPEG Fragment found at pos #" << i + 1 << ". Skipping it" );
           }
         else
@@ -211,7 +213,7 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
       is0.seekg(0);
       SequenceOfFragments sf_bug;
       try {
-        sf_bug.Read<SwapperNoOp>(is0);
+        sf_bug.Read<SwapperNoOp>(is0,true);
       } catch ( ... ) {
         return false;
       }
@@ -239,9 +241,13 @@ bool JPEGCodec::Decode(DataElement const &in, DataElement &out)
       }
     }
   //assert( pos == len );
-  std::string str = os.str();
-  VL::Type strSize = (VL::Type)str.size();
-  out.SetByteValue( &str[0], strSize );
+  const size_t sizeOfOs = os.tellp();
+  os.seekp( 0, std::ios::beg );
+  ByteValue * bv = new ByteValue;
+  bv->SetLength( (uint32_t)sizeOfOs );
+  bv->Read<SwapperNoOp>( os );
+  out.SetValue( *bv );
+
   return true;
 }
 
@@ -377,6 +383,7 @@ bool JPEGCodec::DecodeByStreams(std::istream &is, std::ostream &os)
       if( Internal )
         {
         //Internal->SetPixelFormat( this->GetPixelFormat() ); // FIXME
+        Internal->SetDimensions( this->GetDimensions() );
         Internal->SetPlanarConfiguration( this->GetPlanarConfiguration() ); // meaningless ?
         Internal->SetPhotometricInterpretation( this->GetPhotometricInterpretation() );
         if( Internal->DecodeByStreams(is,tmpos) )
@@ -464,7 +471,7 @@ bool JPEGCodec::DecodeExtent(
     size_t buf_size = 0;
 
     const Tag seqDelItem(0xfffe,0xe0dd);
-    gdcm::Fragment frag;
+    Fragment frag;
     unsigned int nfrags = 0;
     try
       {
@@ -558,7 +565,7 @@ bool JPEGCodec::DecodeExtent(
   else if ( NumberOfDimensions == 3 )
     {
     const Tag seqDelItem(0xfffe,0xe0dd);
-    gdcm::Fragment frag;
+    Fragment frag;
     std::streamoff thestart = is.tellg();
     unsigned int numfrags = 0;
     std::vector< size_t > offsets;
@@ -629,4 +636,55 @@ bool JPEGCodec::IsStateSuspension() const
   return false;
 }
 
+ImageCodec * JPEGCodec::Clone() const
+{
+  JPEGCodec *copy = new JPEGCodec;
+  ImageCodec &ic = *copy;
+  ic = *this;
+  assert( copy->PF == PF );
+  //copy->SetupJPEGBitCodec( BitSample );
+  copy->SetPixelFormat( GetPixelFormat() );
+  assert( copy->BitSample == BitSample || BitSample == 0 );
+  //copy->Lossless = Lossless;
+  copy->Quality = Quality;
+
+  return copy;
+}
+
+bool JPEGCodec::EncodeBuffer( std::ostream & out,
+    const char *inbuffer, size_t inlen)
+{
+  assert( Internal );
+  return Internal->EncodeBuffer(out, inbuffer, inlen);
+}
+
+bool JPEGCodec::StartEncode( std::ostream & )
+{
+  return true;
+}
+bool JPEGCodec::IsRowEncoder()
+{
+  return true;
+}
+bool JPEGCodec::IsFrameEncoder()
+{
+  assert(0);
+  return false;
+}
+bool JPEGCodec::AppendRowEncode( std::ostream & os, const char * data, size_t datalen)
+{
+  return EncodeBuffer(os, data, datalen );
+}
+// TODO: technically the frame encoder could use the row encoder when present
+// this could reduce code duplication
+bool JPEGCodec::AppendFrameEncode( std::ostream & , const char * , size_t )
+{
+  assert(0);
+  return false;
+}
+bool JPEGCodec::StopEncode( std::ostream & )
+{
+  return true;
+}
+
 } // end namespace gdcm
