@@ -103,11 +103,19 @@ void Overlay::Update(const DataElement & de)
 {
 /*
   8.1.2 Overlay data encoding of related data elements
-    Encoded Overlay Planes always have a bit depth of 1, and are encoded separately from the Pixel Data in Overlay Data (60xx,3000). The following two Data Elements shall define the Overlay Plane structure:
-     Overlay Bits Allocated (60xx,0100)
-     Overlay Bit Position (60xx,0102)
-    Notes: 1. There is no Data Element analogous to Bits Stored (0028,0101) since Overlay Planes always have a bit depth of 1.
-    2. Restrictions on the allowed values for these Data Elements are defined in PS 3.3. Formerly overlay data stored in unused bits of Pixel Data (7FE0,0010) was described, and these attributes had meaningful values but this usage has been retired. See PS 3.5 2004. For overlays encoded in Overlay Data Element (60xx,3000), Overlay Bits Allocated (60xx,0100) is always 1 and Overlay Bit Position (60xx,0102) is always 0.
+    Encoded Overlay Planes always have a bit depth of 1, and are encoded
+    separately from the Pixel Data in Overlay Data (60xx,3000). The following two
+    Data Elements shall define the Overlay Plane structure:
+    - Overlay Bits Allocated (60xx,0100)
+    - Overlay Bit Position (60xx,0102)
+    Notes: 1. There is no Data Element analogous to Bits Stored (0028,0101)
+    since Overlay Planes always have a bit depth of 1.
+    2. Restrictions on the allowed values for these Data Elements are defined
+    in PS 3.3. Formerly overlay data stored in unused bits of Pixel Data
+    (7FE0,0010) was described, and these attributes had meaningful values but this
+    usage has been retired. See PS 3.5 2004. For overlays encoded in Overlay Data
+    Element (60xx,3000), Overlay Bits Allocated (60xx,0100) is always 1 and Overlay
+    Bit Position (60xx,0102) is always 0.
 */
 
   assert( de.GetTag().IsPublic() );
@@ -428,64 +436,6 @@ const ByteValue &Overlay::GetOverlayData() const
   return bv;
 }
 
-#if !defined(GDCM_LEGACY_REMOVE)
-void Overlay::Decode(std::istream &is, std::ostream &os)
-{
-  unsigned char packedbytes;
-  unsigned char unpackedbytes[8];
-  while( is.read((char*)&packedbytes,1) )
-    {
-    unsigned char mask = 1;
-    for (unsigned int i = 0; i < 8; ++i)
-      {
-      if ( (packedbytes & mask) == 0)
-        {
-        unpackedbytes[i] = 0;
-        }
-      else
-        {
-        unpackedbytes[i] = 1;
-        }
-      mask <<= 1;
-      }
-    os.write(reinterpret_cast<char*>(unpackedbytes), 8);
-    }
-}
-
-bool Overlay::GetBuffer(char *buffer) const
-{
-  const size_t length = Internal->Data.size();
-  std::copy(buffer, buffer+length, Internal->Data.begin());
-  return true;
-}
-
-bool Overlay::GetUnpackBuffer(unsigned char *buffer) const
-{
-  unsigned char *unpackedbytes = buffer;
-  for( std::vector<char>::const_iterator it = Internal->Data.begin(); it != Internal->Data.end(); ++it )
-    {
-    // const unsigned char &packedbytes = *it;
-    // weird bug with gcc 3.3 (prerelease on SuSE) apparently:
-    unsigned char packedbytes = static_cast<unsigned char>(*it);
-    unsigned char mask = 1;
-    for (unsigned int i = 0; i < 8; ++i)
-      {
-      if ( (packedbytes & mask) == 0)
-        {
-        *unpackedbytes = 0;
-        }
-      else
-        {
-        *unpackedbytes = 255;
-        }
-      ++unpackedbytes;
-      mask <<= 1;
-      }
-    }
-  return true;
-}
-#endif
-
 size_t Overlay::GetUnpackBufferLength() const
 {
   const size_t unpacklen = Internal->Rows * Internal->Columns;
@@ -527,7 +477,7 @@ void Overlay::Decompress(std::ostream &os) const
 {
   const size_t unpacklen = GetUnpackBufferLength();
   unsigned char unpackedbytes[8];
-  std::vector<char>::const_iterator beg = Internal->Data.begin();
+  //std::vector<char>::const_iterator beg = Internal->Data.begin();
   size_t curlen = 0;
   for( std::vector<char>::const_iterator it = Internal->Data.begin(); it != Internal->Data.end(); ++it )
     {
