@@ -720,8 +720,16 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
       photometricinterpretation->GetPointer(),
       photometricinterpretation->GetLength() );
     pi = PhotometricInterpretation::GetPIType( photometricinterpretation_str.c_str() );
+    // http://www.dominator.com/assets/003/5278.pdf
+    // JPEG 2000 lossless YUV_RCT
+    if( pi == PhotometricInterpretation::PI_END )
+      {
+      gdcmWarningMacro( "Discarding suspicious PhotometricInterpretation found: "
+        << photometricinterpretation_str );
+      }
     }
-  else
+  // try again harder:
+  if( !photometricinterpretation || pi == PhotometricInterpretation::PI_END )
     {
     if( pf.GetSamplesPerPixel() == 1 )
       {
@@ -735,11 +743,16 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
       }
     else if( pf.GetSamplesPerPixel() == 4 )
       {
-      gdcmWarningMacro( "No PhotometricInterpretation found, default to RGB" );
+      gdcmWarningMacro( "No PhotometricInterpretation found, default to ARGB" );
       pi = PhotometricInterpretation::ARGB;
       }
+    else
+      {
+      gdcmWarningMacro( "Impossible value for Samples Per Pixel: " << pf.GetSamplesPerPixel() );
+      return false;
+      }
     }
-
+  assert( pi != PhotometricInterpretation::PI_END );
   if( !pf.GetSamplesPerPixel() || (pi.GetSamplesPerPixel() != pf.GetSamplesPerPixel()) )
     {
     if( pi != PhotometricInterpretation::UNKNOW )
@@ -755,7 +768,9 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
       }
     else
       {
-      gdcmWarningMacro( "Cannot recognize image type. Does not looks like ACR-NEMA and is missing both Sample Per Pixel AND PhotometricInterpretation. Please report" );
+      gdcmWarningMacro( "Cannot recognize image type. Does not looks like"
+        "ACR-NEMA and is missing both Sample Per Pixel AND PhotometricInterpretation."
+        "Please report" );
       return false;
       }
     }
@@ -987,7 +1002,7 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
     if( dims[0] == 0 || dims[1] == 0 )
       {
       // Pseudo-declared JPEG SC image storage. Let's fix col/row/pf/pi
-      gdcm::JPEGCodec jpeg;
+      JPEGCodec jpeg;
       if( jpeg.CanDecode( PixelData->GetTransferSyntax() ) )
         {
         std::stringstream ss;
@@ -1003,9 +1018,9 @@ bool PixmapReader::ReadImageInternal(MediaStorage const &ms, bool handlepixeldat
         sqf->WriteBuffer( ss );
         //std::string s( bv->GetPointer(), bv->GetLength() );
         //is.str( s );
-        gdcm::PixelFormat jpegpf ( gdcm::PixelFormat::UINT8 ); // usual guess...
+        PixelFormat jpegpf ( PixelFormat::UINT8 ); // usual guess...
         jpeg.SetPixelFormat( jpegpf );
-        gdcm::TransferSyntax ts;
+        TransferSyntax ts;
         bool b = jpeg.GetHeaderInfo( ss, ts );
         if( b )
           {
