@@ -332,8 +332,8 @@ bool ULConnectionManager::EstablishConnectionMove(const std::string& inAETitle,
 
   if (Trace::GetDebugFlag())
     {
-      vector<BasePDU*> thePDUs = theEvent.GetPDUs();
-      vector<BasePDU*>::iterator itor;
+      std::vector<BasePDU*> thePDUs = theEvent.GetPDUs();
+      std::vector<BasePDU*>::iterator itor;
       for (itor = thePDUs.begin(); itor != thePDUs.end(); itor++)
         {
         if (*itor == NULL) continue; //can have a nulled pdu, apparently
@@ -423,7 +423,7 @@ void ULConnectionManager::SendStore(const File & file, ULConnectionCallback* inC
 
   ULEvent theEvent(ePDATArequest, theDataPDU);
   EStateID theState = RunEventLoop(theEvent, mConnection, inCallback, false);
-  assert( theState == eSta6TransferReady ); (void)theState;
+  assert( theState == eSta6TransferReady || theState == eStaDoesNotExist ); (void)theState;
 }
 
 bool ULConnectionManager::BreakConnection(const double& inTimeOut){
@@ -571,7 +571,7 @@ EStateID ULConnectionManager::RunMoveEventLoop(ULEvent& currentEvent, ULConnecti
             Attribute<0x0,0x0800> at;
             at.SetFromDataElement( de );
             unsigned short datasettype = at.GetValue();
-            assert( datasettype == 0x0101 );
+            assert( datasettype == 0x0101 || datasettype == 0x1 ); (void)datasettype;
           }
           if (theRSP.FindDataElement(Tag(0x0, 0x0900))){
             DataElement const & de = theRSP.GetDataElement(Tag(0x0,0x0900));
@@ -647,12 +647,12 @@ EStateID ULConnectionManager::RunMoveEventLoop(ULEvent& currentEvent, ULConnecti
             if (theRSP.FindDataElement(Tag(0x0,0x0901))){
               DataElement de = theRSP.GetDataElement(Tag(0x0,0x0901));
               err1 = de.GetByteValue();
-              gdcmErrorMacro( " Tag 0x0,0x901 reported as " << *err1 << std::endl);
+              gdcmErrorMacro( " Tag 0x0,0x901 reported as " << *err1 << std::endl); (void)err1;
             }
             if (theRSP.FindDataElement(Tag(0x0,0x0902))){
               DataElement de = theRSP.GetDataElement(Tag(0x0,0x0902));
               err2 = de.GetByteValue();
-              gdcmErrorMacro( " Tag 0x0,0x902 reported as " << *err2 << std::endl);
+              gdcmErrorMacro( " Tag 0x0,0x902 reported as " << *err2 << std::endl); (void)err2;
             }
           }
           receivingData = false;
@@ -946,12 +946,12 @@ EStateID ULConnectionManager::RunEventLoop(ULEvent& currentEvent, ULConnection*
                 if (theRSP.FindDataElement(Tag(0x0,0x0901))){
                   DataElement de = theRSP.GetDataElement(Tag(0x0,0x0901));
                   err1 = de.GetByteValue();
-                  gdcmErrorMacro( " Tag 0x0,0x901 reported as " << *err1 << std::endl);
+                  gdcmErrorMacro( " Tag 0x0,0x901 reported as " << *err1 << std::endl); (void)err1;
                 }
                 if (theRSP.FindDataElement(Tag(0x0,0x0902))){
                   DataElement de = theRSP.GetDataElement(Tag(0x0,0x0902));
                   err2 = de.GetByteValue();
-                  gdcmErrorMacro( " Tag 0x0,0x902 reported as " << *err2 << std::endl);
+                  gdcmErrorMacro( " Tag 0x0,0x902 reported as " << *err2 << std::endl); (void)err2;
                 }
               }
 
@@ -994,8 +994,30 @@ EStateID ULConnectionManager::RunEventLoop(ULEvent& currentEvent, ULConnection*
                   //!!!need to handle incoming PDUs that are not data, ie, an abort
                 } while(!thePDU->IsLastFragment());
                 if (!interrupted){//ie, if the remote server didn't hang up
-                  DataSet theCompleteFindResponse =
-                    PresentationDataValue::ConcatenatePDVBlobs(PDUFactory::GetPDVs(theData));
+                  bool useimplicit = true;
+                  TransferSyntaxSub ts1;
+                  ts1.SetNameFromUID( UIDs::ImplicitVRLittleEndianDefaultTransferSyntaxforDICOM );
+                  if( mSecondaryConnection )
+                    {
+                    const TransferSyntaxSub & ts_ = mSecondaryConnection->GetCStoreTransferSyntax();
+                    if( strcmp(ts_.GetName(), ts1.GetName()) != 0)
+                      {
+                      useimplicit = false;
+                      }
+                    }
+                  DataSet theCompleteFindResponse;
+                  if( useimplicit )
+                    {
+                    inCallback->SetImplicitFlag(true);
+                    theCompleteFindResponse =
+                      PresentationDataValue::ConcatenatePDVBlobs(PDUFactory::GetPDVs(theData));
+                    }
+                  else
+                    {
+                    inCallback->SetImplicitFlag(false);
+                    theCompleteFindResponse =
+                      PresentationDataValue::ConcatenatePDVBlobsAsExplicit(PDUFactory::GetPDVs(theData));
+                    }
                   //note that it's the responsibility of the event to delete the PDU in theFindRSP
                   for (size_t i = 0; i < theData.size(); i++)
                     {
@@ -1045,6 +1067,11 @@ EStateID ULConnectionManager::RunEventLoop(ULEvent& currentEvent, ULConnection*
               waitingForEvent = false;
               receivingData = true; //to continue the loop to process the release
               break;
+            case eAABORTPDUReceivedOpen:
+              {
+              raisedEvent = eEventDoesNotExist;
+              theState = eStaDoesNotExist;
+              } // explicitely declare fall-through for some picky compiler
             case eAABORTRequest:
               waitingForEvent = false;
               inWhichConnection->StopProtocol();
