@@ -279,14 +279,21 @@ void vtkGDCMImageWriter::Write()
     vtkErrorMacro("Write: No input supplied.");
     return;
     }
-
+#if (VTK_MAJOR_VERSION >= 6)
+#else
   input->UpdateInformation();
+#endif
 
   // Update the rest.
   this->UpdateInformation();
 
   // Get the whole extent of the input
+#if (VTK_MAJOR_VERSION >= 6)
+  vtkInformation *inInfo = this->GetInputInformation(0, 0);
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), this->DataUpdateExtent);
+#else
   input->GetWholeExtent(this->DataUpdateExtent);
+#endif
 
   if (this->DataUpdateExtent[0] == (this->DataUpdateExtent[1] + 1) ||
       this->DataUpdateExtent[2] == (this->DataUpdateExtent[3] + 1) ||
@@ -446,10 +453,18 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
 {
   //std::cerr << "Calling WriteGDCMData" << std::endl;
   assert( timeStep >= 0 );
+#if (VTK_MAJOR_VERSION >= 6)
+  vtkInformation *inInfo = this->GetInputInformation(0, timeStep);
+  int inWholeExt[6];
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), inWholeExt);
+  int inExt[6];
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), inExt);
+#else
   int inWholeExt[6];
   data->GetWholeExtent(inWholeExt);
   int inExt[6];
   data->GetUpdateExtent(inExt);
+#endif
 #if (VTK_MAJOR_VERSION >= 5) || ( VTK_MAJOR_VERSION == 4 && VTK_MINOR_VERSION > 2 )
   vtkIdType inInc[3];
 #else
@@ -505,6 +520,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
   //this->FileDimensionality
   int scalarType = data->GetScalarType();
   gdcm::PixelFormat pixeltype = gdcm::PixelFormat::UNKNOWN;
+  bool forcerescale = false;
   switch( scalarType )
     {
   case VTK_BIT:
@@ -548,6 +564,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
     will be doing unsigned short anyway...
     */
     pixeltype = gdcm::PixelFormat::FLOAT32;
+    forcerescale = true;
     break;
   case VTK_DOUBLE:
     if( this->Shift == (int)this->Shift && this->Scale == (int)this->Scale )
@@ -561,6 +578,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
     will be doing unsigned short anyway...
     */
     pixeltype = gdcm::PixelFormat::FLOAT64;
+    forcerescale = true;
     break;
   default:
     vtkErrorMacro( "Do not support this Pixel Type: " << scalarType );
@@ -626,7 +644,10 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
 
   // store in a safe place the 'raw' pixeltype from vtk
   gdcm::PixelFormat savepixeltype = pixeltype;
-  if( this->Shift == 0 && this->Scale == 1 )
+  // fast path, when (shift,scale) is (0,1) we do not need no rescale function
+  // at all. Pay attention in some case users really wants to store floating
+  // point values anyway, be nice with them
+  if( !forcerescale && (this->Shift == 0 && this->Scale == 1) )
     {
     //assert( pixeltype == outputpt );
     }
@@ -727,7 +748,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
   gdcm::ByteValue *bv = new gdcm::ByteValue(); // (char*)data->GetScalarPointer(), len );
   bv->SetLength( (uint32_t)len ); // allocate !
 
-//  std::ofstream of( "/tmp/bla.raw" );
+//  std::ofstream of( "/tmp/bla.raw", std::ios::binary );
 //  of.write( (char*)data->GetScalarPointer(), len);
 //  of.close();
   // re shuffle the line within ByteValue:
@@ -752,7 +773,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
   bool rescaled = false;
   char * copy = NULL;
   // Whenever shift / scale is needed... do it !
-  if( this->Shift != 0 || this->Scale != 1 )
+  if( this->Shift != 0 || this->Scale != 1 || forcerescale )
     {
     assert( this->PlanarConfiguration == 0 );
     // rescale from float to unsigned short
@@ -864,14 +885,14 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
     SetStringValueFromTag( this->MedicalImageProperties->GetPatientSex(), gdcm::Tag(0x0010,0x0040), ano);
     // For ex: DICOM (0010,0030) = 19680427
     SetStringValueFromTag( this->MedicalImageProperties->GetPatientBirthDate(), gdcm::Tag(0x0010,0x0030), ano);
-#if ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
+#if VTK_MAJOR_VERSION >= 6 || ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
     // For ex: DICOM (0008,0020) = 20030617
     if( vtkMedicalImageProperties::GetDateAsFields( this->MedicalImageProperties->GetStudyDate(), year, month, day ) )
       SetStringValueFromTag( this->MedicalImageProperties->GetStudyDate(), gdcm::Tag(0x0008,0x0020), ano);
 #endif
     // For ex: DICOM (0008,0022) = 20030617
     SetStringValueFromTag( this->MedicalImageProperties->GetAcquisitionDate(), gdcm::Tag(0x0008,0x0022), ano);
-#if ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
+#if VTK_MAJOR_VERSION >= 6 || ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
     // For ex: DICOM (0008,0030) = 162552.0705 or 230012, or 0012
 #if 0
     int hour, minute, second;
@@ -970,7 +991,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
         ds.Insert( de );
         }
       }
-#if ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
+#if VTK_MAJOR_VERSION >= 6 || ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 0 )
     // User defined value
     // Remap any user defined value from the DICOM name to the DICOM tag
     unsigned int nvalues = this->MedicalImageProperties->GetNumberOfUserDefinedValues();
@@ -991,99 +1012,16 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
 #endif
     }
 
-
-  if( this->FileDimensionality != 2 && (
-      ms == gdcm::MediaStorage::SecondaryCaptureImageStorage ||
-      ms == gdcm::MediaStorage::MultiframeSingleBitSecondaryCaptureImageStorage
-  ) )
+  ms = gdcm::ImageHelper::ComputeMediaStorageFromModality(
+    this->MedicalImageProperties->GetModality(), this->FileDimensionality,
+    pixeltype, pi, this->Shift, this->Scale );
+  if( ms == gdcm::MediaStorage::MS_END )
     {
-    // A.8.3.4 Multi-frame Grayscale Byte SC Image IOD Content Constraints
-/*
-- Samples per Pixel (0028,0002) shall be 1
-- Photometric Interpretation (0028,0004) shall be MONOCHROME2
-- Bits Allocated (0028,0100) shall be 8
-- Bits Stored (0028,0101) shall be 8
-- High Bit (0028,0102) shall be 7
-- Pixel Representation (0028,0103) shall be 0
-- Planar Configuration (0028,0006) shall not be present
-*/
-    if( this->FileDimensionality == 3 &&
-      pixeltype.GetSamplesPerPixel() == 1 &&
-      pi == gdcm::PhotometricInterpretation::MONOCHROME2 &&
-      pixeltype.GetBitsAllocated() == 8 &&
-      pixeltype.GetBitsStored() == 8 &&
-      pixeltype.GetHighBit() == 7 &&
-      pixeltype.GetPixelRepresentation() == 0
-      // image.GetPlanarConfiguration()
-    )
-      {
-      ms = gdcm::MediaStorage::MultiframeGrayscaleByteSecondaryCaptureImageStorage;
-      if( this->Shift != 0 || this->Scale != 1 )
-        {
-        // Table C.8-25b SC MULTI-FRAME IMAGE MODULE ATTRIBUTES
-        // Note: This specifies an identity Modality LUT transformation.
-        vtkErrorMacro( "Cannot have shift/scale" );
-        return 0;
-        }
-      }
-    else if( this->FileDimensionality == 3 &&
-      pixeltype.GetSamplesPerPixel() == 1 &&
-      pi == gdcm::PhotometricInterpretation::MONOCHROME2 &&
-      pixeltype.GetBitsAllocated() == 1 &&
-      pixeltype.GetBitsStored() == 1 &&
-      pixeltype.GetHighBit() == 0 &&
-      pixeltype.GetPixelRepresentation() == 0
-      // image.GetPlanarConfiguration()
-    )
-      {
-      ms = gdcm::MediaStorage::MultiframeSingleBitSecondaryCaptureImageStorage;
-      if( this->Shift != 0 || this->Scale != 1 )
-        {
-        vtkErrorMacro( "Cannot have shift/scale" );
-        return 0;
-        }
-      }
-    else if( this->FileDimensionality == 3 &&
-      pixeltype.GetSamplesPerPixel() == 1 &&
-      pi == gdcm::PhotometricInterpretation::MONOCHROME2 &&
-      pixeltype.GetBitsAllocated() == 16 &&
-      pixeltype.GetBitsStored() <= 16 && pixeltype.GetBitsStored() >= 9 &&
-      pixeltype.GetHighBit() == pixeltype.GetBitsStored() - 1 &&
-      pixeltype.GetPixelRepresentation() == 0
-      // image.GetPlanarConfiguration()
-    )
-      {
-      ms = gdcm::MediaStorage::MultiframeGrayscaleWordSecondaryCaptureImageStorage;
-      if( this->Shift != 0 || this->Scale != 1 )
-        {
-        vtkErrorMacro( "Cannot have shift/scale" );
-        return 0;
-        }
-      }
-    else if( this->FileDimensionality == 3 &&
-      pixeltype.GetSamplesPerPixel() == 3 &&
-      pi == gdcm::PhotometricInterpretation::RGB &&
-      pixeltype.GetBitsAllocated() == 8 &&
-      pixeltype.GetBitsStored() == 8 &&
-      pixeltype.GetHighBit() == 7 &&
-      pixeltype.GetPixelRepresentation() == 0
-      // image.GetPlanarConfiguration()
-    )
-      {
-      ms = gdcm::MediaStorage::MultiframeTrueColorSecondaryCaptureImageStorage;
-      if( this->Shift != 0 || this->Scale != 1 )
-        {
-        vtkErrorMacro( "Cannot have shift/scale" );
-        return 0;
-        }
-      }
-    else
-      {
-      vtkErrorMacro( "Cannot handle Multi Frame image in SecondaryCaptureImageStorage" );
-      return 0;
-      }
+    vtkErrorMacro( "Incompatible MediaStorage" );
+    return 0;
     }
 
+
   // FIXME: new Secondary object handle multi frames...
   assert( gdcm::MediaStorage::IsImage( ms ) );
     {
@@ -1115,7 +1053,7 @@ int vtkGDCMImageWriter::WriteGDCMData(vtkImageData *data, int timeStep)
     {
     iop[i+3] = dircos->GetElement(i,1);
     }
-#if ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 2 )
+#if VTK_MAJOR_VERSION >= 6 || ( VTK_MAJOR_VERSION == 5 && VTK_MINOR_VERSION > 2 )
    const double *iop_mip = this->MedicalImageProperties->GetDirectionCosine();
    if( iop[0] != iop_mip[0]
     || iop[1] != iop_mip[1]
